@@ -145,6 +145,7 @@ typedef struct {
 } QueuedStatement;
 
 static gboolean in_transaction = FALSE;
+static gboolean in_ontology_transaction = FALSE;
 static gboolean in_journal_replay = FALSE;
 static TrackerDataUpdateBuffer update_buffer;
 /* current resource */
@@ -157,6 +158,7 @@ static GPtrArray *delete_callbacks = NULL;
 static GPtrArray *commit_callbacks = NULL;
 static GPtrArray *rollback_callbacks = NULL;
 static gint max_service_id = 0;
+static gint max_ontology_id = 0;
 static gint max_modseq = 0;
 
 static gint ensure_resource_id (const gchar *uri, gboolean    *create);
@@ -338,40 +340,74 @@ tracker_data_update_get_new_service_id (void)
 	TrackerDBStatement *stmt;
 	GError *error = NULL;
 
-	if (G_LIKELY (max_service_id != 0)) {
-		return ++max_service_id;
-	}
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT, &error,
-	                                              "SELECT MAX(ID) AS A FROM Resource");
-
-	if (stmt) {
-		cursor = tracker_db_statement_start_cursor (stmt, &error);
-		g_object_unref (stmt);
-	}
-
-	if (cursor) {
-		if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
-			max_service_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_service_id);
+	if (in_ontology_transaction) {
+		if (G_LIKELY (max_ontology_id != 0)) {
+			return ++max_ontology_id;
 		}
 
-		g_object_unref (cursor);
-	}
+		iface = tracker_db_manager_get_db_interface ();
 
-	if (G_UNLIKELY (error)) {
-		g_warning ("Could not get new resource ID: %s\n", error->message);
-		g_error_free (error);
-	}
+		stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT, &error,
+		                                              "SELECT MAX(ID) AS A FROM Resource WHERE ID <= %d", TRACKER_ONTOLOGIES_MAX_ID);
 
-	return ++max_service_id;
+		if (stmt) {
+			cursor = tracker_db_statement_start_cursor (stmt, &error);
+			g_object_unref (stmt);
+		}
+
+		if (cursor) {
+			if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
+				max_ontology_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_ontology_id);
+			}
+
+			g_object_unref (cursor);
+		}
+
+		if (G_UNLIKELY (error)) {
+			g_warning ("Could not get new resource ID for ontology transaction: %s\n", error->message);
+			g_error_free (error);
+		}
+
+		return ++max_ontology_id;
+	} else {
+		if (G_LIKELY (max_service_id != 0)) {
+			return ++max_service_id;
+		}
+
+		max_service_id = TRACKER_ONTOLOGIES_MAX_ID;
+
+		iface = tracker_db_manager_get_db_interface ();
+
+		stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT, &error,
+		                                              "SELECT MAX(ID) AS A FROM Resource");
+
+		if (stmt) {
+			cursor = tracker_db_statement_start_cursor (stmt, &error);
+			g_object_unref (stmt);
+		}
+
+		if (cursor) {
+			if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
+				max_service_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_service_id);
+			}
+
+			g_object_unref (cursor);
+		}
+
+		if (G_UNLIKELY (error)) {
+			g_warning ("Could not get new resource ID: %s\n", error->message);
+			g_error_free (error);
+		}
+
+		return ++max_service_id;
+	}
 }
 
 void
 tracker_data_update_shutdown (void)
 {
 	max_service_id = 0;
+	max_ontology_id = 0;
 	max_modseq = 0;
 }
 
@@ -2219,7 +2255,11 @@ tracker_data_begin_transaction (GError **error)
 	tracker_db_interface_start_transaction (iface);
 
 	if (!in_journal_replay) {
-		tracker_db_journal_start_transaction (resource_time);
+		if (in_ontology_transaction) {
+			tracker_db_journal_start_ontology_transaction (resource_time);
+		} else {
+			tracker_db_journal_start_transaction (resource_time);
+		}
 	}
 
 	iface = tracker_db_manager_get_db_interface ();
@@ -2227,6 +2267,13 @@ tracker_data_begin_transaction (GError **error)
 	in_transaction = TRUE;
 }
 
+void
+tracker_data_begin_ontology_transaction (GError **error)
+{
+	in_ontology_transaction = TRUE;
+	tracker_data_begin_transaction (error);
+}
+
 void
 tracker_data_begin_transaction_for_replay (time_t time, GError **error)
 {
@@ -2252,6 +2299,7 @@ tracker_data_commit_transaction (GError **error)
 	}
 
 	in_transaction = FALSE;
+	in_ontology_transaction = FALSE;
 
 	if (!in_journal_replay) {
 		tracker_db_journal_commit_db_transaction ();
@@ -2562,6 +2610,7 @@ tracker_data_rollback_transaction (void)
 	g_return_if_fail (in_transaction);
 
 	in_transaction = FALSE;
+	in_ontology_transaction = FALSE;
 
 	iface = tracker_db_manager_get_db_interface ();
 
@@ -2650,161 +2699,15 @@ tracker_data_sync (void)
 	tracker_db_journal_fsync ();
 }
 
-static void
-free_queued_statement (QueuedStatement *queued)
-{
-	g_free (queued->subject);
-	g_free (queued->predicate);
-	g_free (queued->object);
-	g_free (queued->graph);
-	g_free (queued);
-}
-
-static GList*
-queue_statement (GList *queue,
-                 const gchar *graph,
-                 const gchar *subject,
-                 const gchar *predicate,
-                 const gchar *object,
-                 gboolean     is_uri)
-{
-	QueuedStatement *queued = g_new (QueuedStatement, 1);
-
-	queued->subject = g_strdup (subject);
-	queued->predicate = g_strdup (predicate);
-	queued->object = g_strdup (object);
-	queued->is_uri = is_uri;
-	queued->graph = graph ? g_strdup (graph) : NULL;
-
-	queue = g_list_append (queue, queued);
-
-	return queue;
-}
-
-static void
-ontology_transaction_end (GList *ontology_queue,
-                          GPtrArray *seen_classes,
-                          GPtrArray *seen_properties)
-{
-	GList *l;
-	const gchar *ontology_uri = NULL;
-
-	tracker_data_ontology_process_changes_pre_db (seen_classes, seen_properties);
-
-	/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
-	tracker_data_ontology_import_into_db (TRUE);
-
-	tracker_data_ontology_process_changes_post_db (seen_classes, seen_properties);
-
-	for (l = ontology_queue; l; l = l->next) {
-		QueuedStatement *queued = ontology_queue->data;
-
-		if (g_strcmp0 (queued->predicate, RDF_TYPE) == 0) {
-			if (g_strcmp0 (queued->object, TRACKER_PREFIX "Ontology") == 0) {
-				ontology_uri = queued->subject;
-			}
-		}
-
-		/* store ontology in database */
-		tracker_data_ontology_process_statement (queued->graph,
-		                                         queued->subject,
-		                                         queued->predicate,
-		                                         queued->object,
-		                                         queued->is_uri,
-		                                         TRUE, TRUE);
-
-	}
-
-	tracker_data_ontology_process_changes_post_import (seen_classes, seen_properties);
-
-	/* Update the nao:lastModified in the database */
-	if (ontology_uri) {
-		TrackerOntology *ontology;
-		ontology = tracker_ontologies_get_ontology_by_uri (ontology_uri);
-		if (ontology) {
-			gint last_mod = 0;
-			TrackerDBInterface *iface;
-			TrackerDBStatement *stmt;
-			GError *error = NULL;
-
-			iface = tracker_db_manager_get_db_interface ();
-
-			/* We can't do better than this cast, it's stored as an int in the
-			 * db. See tracker-data-manager.c for more info. */
-			last_mod = (gint) tracker_ontology_get_last_modified (ontology);
-
-			stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE, &error,
-			        "UPDATE \"rdfs:Resource\" SET \"nao:lastModified\"= ? "
-			        "WHERE \"rdfs:Resource\".ID = "
-			        "(SELECT Resource.ID FROM Resource INNER JOIN \"rdfs:Resource\" "
-			        "ON \"rdfs:Resource\".ID = Resource.ID WHERE "
-			        "Resource.Uri = ?)");
-
-			if (stmt) {
-				tracker_db_statement_bind_int (stmt, 0, last_mod);
-				tracker_db_statement_bind_text (stmt, 1, ontology_uri);
-				tracker_db_statement_execute (stmt, NULL);
-			} else {
-				g_warning ("%s", error->message);
-				g_error_free (error);
-			}
-		}
-	}
-
-	/* Reset the is_new flag for all classes and properties */
-	tracker_data_ontology_import_finished ();
-}
-
-static GList*
-ontology_statement_insert (GList       *ontology_queue,
-                           gint         graph_id,
-                           gint         subject_id,
-                           gint         predicate_id,
-                           const gchar *object,
-                           GHashTable  *classes,
-                           GHashTable  *properties,
-                           GHashTable  *id_uri_map,
-                           gboolean     is_uri,
-                           GPtrArray   *seen_classes,
-                           GPtrArray   *seen_properties)
-{
-	const gchar *graph, *subject, *predicate;
-
-	if (graph_id > 0) {
-		graph = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (graph_id));
-	} else {
-		graph = NULL;
-	}
-
-	subject = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (subject_id));
-	predicate = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (predicate_id));
-
-	/* load ontology from journal into memory, set all new's is_new to TRUE */
-	tracker_data_ontology_load_statement ("journal", subject_id, subject, predicate,
-	                                      object, NULL, TRUE, classes, properties,
-	                                      seen_classes, seen_properties);
-
-	/* Queue the statement for processing after ALTER in ontology_transaction_end */
-	ontology_queue = queue_statement (ontology_queue, graph, subject, predicate, object, is_uri);
-
-	return ontology_queue;
-}
-
 void
-tracker_data_replay_journal (GHashTable          *classes,
-                             GHashTable          *properties,
-                             GHashTable          *id_uri_map,
-                             TrackerBusyCallback  busy_callback,
+tracker_data_replay_journal (TrackerBusyCallback  busy_callback,
                              gpointer             busy_user_data,
                              const gchar         *busy_status)
 {
 	GError *journal_error = NULL;
 	TrackerProperty *rdf_type = NULL;
 	gint last_operation_type = 0;
-	gboolean in_ontology = FALSE;
-	GList *ontology_queue = NULL;
-	GPtrArray *seen_classes = NULL;
-	GPtrArray *seen_properties = NULL;
+	const gchar *uri;
 
 	rdf_type = tracker_ontologies_get_rdf_type ();
 
@@ -2821,31 +2724,13 @@ tracker_data_replay_journal (GHashTable          *classes,
 			TrackerDBInterface *iface;
 			TrackerDBStatement *stmt;
 			gint id;
-			const gchar *uri;
-			TrackerProperty *property = NULL;
-			TrackerClass *class;
 
 			tracker_db_journal_reader_get_resource (&id, &uri);
 
-			if (in_ontology) {
-				g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), g_strdup (uri));
-				continue;
-			}
-
-			class = g_hash_table_lookup (classes, GINT_TO_POINTER (id));
-			if (!class)
-				property = g_hash_table_lookup (properties, GINT_TO_POINTER (id));
-
-			if (property || class)
-				continue;
-
 			iface = tracker_db_manager_get_db_interface ();
 
 			stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE, &new_error,
-					                              "INSERT "
-					                              "INTO Resource "
-					                              "(ID, Uri) "
-					                              "VALUES (?, ?)");
+			                                              "INSERT INTO Resource (ID, Uri) VALUES (?, ?)");
 
 			if (stmt) {
 				tracker_db_statement_bind_int (stmt, 0, id);
@@ -2859,27 +2744,12 @@ tracker_data_replay_journal (GHashTable          *classes,
 				g_error_free (new_error);
 			}
 
-		} else if (type == TRACKER_DB_JOURNAL_START_ONTOLOGY_TRANSACTION) {
-			tracker_data_begin_transaction_for_replay (tracker_db_journal_reader_get_time (), NULL);
-			in_ontology = TRUE;
 		} else if (type == TRACKER_DB_JOURNAL_START_TRANSACTION) {
 			tracker_data_begin_transaction_for_replay (tracker_db_journal_reader_get_time (), NULL);
 		} else if (type == TRACKER_DB_JOURNAL_END_TRANSACTION) {
 			GError *new_error = NULL;
 			tracker_data_update_buffer_might_flush (&new_error);
 
-			if (in_ontology) {
-				ontology_transaction_end (ontology_queue, seen_classes, seen_properties);
-				g_list_foreach (ontology_queue, (GFunc) free_queued_statement, NULL);
-				g_list_free (ontology_queue);
-				ontology_queue = NULL;
-				in_ontology = FALSE;
-				tracker_data_ontology_free_seen (seen_classes);
-				seen_classes = NULL;
-				tracker_data_ontology_free_seen (seen_properties);
-				seen_properties = NULL;
-			}
-
 			tracker_data_commit_transaction (&new_error);
 			if (new_error) {
 				g_warning ("Journal replay error: '%s'", new_error->message);
@@ -2887,31 +2757,10 @@ tracker_data_replay_journal (GHashTable          *classes,
 			}
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
 			GError *new_error = NULL;
-			TrackerProperty *property;
+			TrackerProperty *property = NULL;
 
 			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
 
-			if (in_ontology) {
-
-				if (!seen_classes)
-					seen_classes = g_ptr_array_new ();
-				if (!seen_properties)
-					seen_properties = g_ptr_array_new ();
-
-				ontology_queue = ontology_statement_insert (ontology_queue,
-				                                            graph_id,
-				                                            subject_id,
-				                                            predicate_id,
-				                                            object,
-				                                            classes,
-				                                            properties,
-				                                            id_uri_map,
-				                                            FALSE,
-				                                            seen_classes,
-				                                            seen_properties);
-				continue;
-			}
-
 			if (last_operation_type == -1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -2921,7 +2770,10 @@ tracker_data_replay_journal (GHashTable          *classes,
 			}
 			last_operation_type = 1;
 
-			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
+			uri = tracker_ontologies_get_uri_by_id (predicate_id);
+			if (uri) {
+				property = tracker_ontologies_get_property_by_uri (uri);
+			}
 
 			if (property) {
 				resource_buffer_switch (NULL, graph_id, NULL, subject_id);
@@ -2940,33 +2792,10 @@ tracker_data_replay_journal (GHashTable          *classes,
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT_ID) {
 			GError *new_error = NULL;
 			TrackerClass *class = NULL;
-			TrackerProperty *property;
+			TrackerProperty *property = NULL;
 
 			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
 
-			if (in_ontology) {
-				const gchar *object_n;
-				object_n = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (object_id));
-
-				if (!seen_classes)
-					seen_classes = g_ptr_array_new ();
-				if (!seen_properties)
-					seen_properties = g_ptr_array_new ();
-
-				ontology_queue = ontology_statement_insert (ontology_queue,
-				                                            graph_id,
-				                                            subject_id,
-				                                            predicate_id,
-				                                            object_n,
-				                                            classes,
-				                                            properties,
-				                                            id_uri_map,
-				                                            TRUE,
-				                                            seen_classes,
-				                                            seen_properties);
-				continue;
-			}
-
 			if (last_operation_type == -1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -2976,7 +2805,10 @@ tracker_data_replay_journal (GHashTable          *classes,
 			}
 			last_operation_type = 1;
 
-			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
+			uri = tracker_ontologies_get_uri_by_id (predicate_id);
+			if (uri) {
+				property = tracker_ontologies_get_property_by_uri (uri);
+			}
 
 			if (property) {
 				if (tracker_property_get_data_type (property) != TRACKER_PROPERTY_TYPE_RESOURCE) {
@@ -2985,7 +2817,10 @@ tracker_data_replay_journal (GHashTable          *classes,
 					resource_buffer_switch (NULL, graph_id, NULL, subject_id);
 
 					if (property == rdf_type) {
-						class = g_hash_table_lookup (classes, GINT_TO_POINTER (object_id));
+						uri = tracker_ontologies_get_uri_by_id (object_id);
+						if (uri) {
+							class = tracker_ontologies_get_class_by_uri (uri);
+						}
 						if (class) {
 							cache_create_service_decomposed (class, NULL, graph_id);
 						} else {
@@ -3009,14 +2844,10 @@ tracker_data_replay_journal (GHashTable          *classes,
 
 		} else if (type == TRACKER_DB_JOURNAL_DELETE_STATEMENT) {
 			GError *new_error = NULL;
-			TrackerProperty *property;
+			TrackerProperty *property = NULL;
 
 			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
 
-			if (in_ontology) {
-				continue;
-			}
-
 			if (last_operation_type == 1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -3028,15 +2859,21 @@ tracker_data_replay_journal (GHashTable          *classes,
 
 			resource_buffer_switch (NULL, graph_id, NULL, subject_id);
 
-			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
+			uri = tracker_ontologies_get_uri_by_id (predicate_id);
+			if (uri) {
+				property = tracker_ontologies_get_property_by_uri (uri);
+			}
 
 			if (property) {
 				GError *new_error = NULL;
 
 				if (object && rdf_type == property) {
-					TrackerClass *class;
+					TrackerClass *class = NULL;
 
-					class = tracker_ontologies_get_class_by_uri (object);
+					uri = tracker_ontologies_get_uri_by_id (object_id);
+					if (uri) {
+						class = tracker_ontologies_get_class_by_uri (uri);
+					}
 					if (class != NULL) {
 						cache_delete_resource_type (class, NULL, graph_id);
 					} else {
@@ -3058,14 +2895,10 @@ tracker_data_replay_journal (GHashTable          *classes,
 		} else if (type == TRACKER_DB_JOURNAL_DELETE_STATEMENT_ID) {
 			GError *new_error = NULL;
 			TrackerClass *class = NULL;
-			TrackerProperty *property;
+			TrackerProperty *property = NULL;
 
 			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
 
-			if (in_ontology) {
-				continue;
-			}
-
 			if (last_operation_type == 1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -3075,14 +2908,20 @@ tracker_data_replay_journal (GHashTable          *classes,
 			}
 			last_operation_type = -1;
 
-			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
+			uri = tracker_ontologies_get_uri_by_id (predicate_id);
+			if (uri) {
+				property = tracker_ontologies_get_property_by_uri (uri);
+			}
 
 			if (property) {
 
 				resource_buffer_switch (NULL, graph_id, NULL, subject_id);
 
 				if (property == rdf_type) {
-					class = g_hash_table_lookup (classes, GINT_TO_POINTER (object_id));
+					uri = tracker_ontologies_get_uri_by_id (object_id);
+					if (uri) {
+						class = tracker_ontologies_get_class_by_uri (uri);
+					}
 					if (class) {
 						cache_delete_resource_type (class, NULL, graph_id);
 					} else {
