@@ -26,17 +26,12 @@
 #include <stdlib.h>
 #include <math.h>
 
-/* I don't know why, but this prototype ain't in my math.h */
-long long int llroundl(long double x);
-
 #include <glib.h>
 #include <glib/gstdio.h>
 
 #include <gst/gst.h>
 #include <gst/tag/tag.h>
 
-#include <libtracker-client/tracker.h>
-
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-albumart.h"
@@ -161,9 +156,9 @@ const gboolean use_tagreadbin = FALSE;
 
 static void
 add_int64_info (TrackerSparqlBuilder *metadata,
-                const gchar *uri,
-                const gchar *key,
-                gint64      info)
+                const gchar          *uri,
+                const gchar          *key,
+                gint64                info)
 {
 	tracker_sparql_builder_predicate (metadata, key);
 	tracker_sparql_builder_object_int64 (metadata, info);
@@ -171,9 +166,9 @@ add_int64_info (TrackerSparqlBuilder *metadata,
 
 static void
 add_uint_info (TrackerSparqlBuilder *metadata,
-               const gchar *uri,
-               const gchar *key,
-               guint       info)
+               const gchar          *uri,
+               const gchar          *key,
+               guint                 info)
 {
 	tracker_sparql_builder_predicate (metadata, key);
 	tracker_sparql_builder_object_int64 (metadata, info);
@@ -203,11 +198,11 @@ add_string_gst_tag (TrackerSparqlBuilder *metadata,
 }
 
 static void
-add_uint_gst_tag (TrackerSparqlBuilder   *metadata,
-                  const gchar *uri,
-                  const gchar *key,
-                  GstTagList  *tag_list,
-                  const gchar *tag)
+add_uint_gst_tag (TrackerSparqlBuilder  *metadata,
+                  const gchar           *uri,
+                  const gchar           *key,
+                  GstTagList            *tag_list,
+                  const gchar           *tag)
 {
 	gboolean ret;
 	guint    n;
@@ -221,11 +216,11 @@ add_uint_gst_tag (TrackerSparqlBuilder   *metadata,
 }
 
 static void
-add_int_gst_tag (TrackerSparqlBuilder   *metadata,
-                 const gchar *uri,
-                 const gchar *key,
-                 GstTagList  *tag_list,
-                 const gchar *tag)
+add_int_gst_tag (TrackerSparqlBuilder  *metadata,
+                 const gchar           *uri,
+                 const gchar           *key,
+                 GstTagList            *tag_list,
+                 const gchar           *tag)
 {
 	gboolean ret;
 	gint     n;
@@ -239,11 +234,11 @@ add_int_gst_tag (TrackerSparqlBuilder   *metadata,
 }
 
 static void
-add_double_gst_tag (TrackerSparqlBuilder        *metadata,
-                    const gchar *uri,
-                    const gchar *key,
-                    GstTagList  *tag_list,
-                    const gchar *tag)
+add_double_gst_tag (TrackerSparqlBuilder  *metadata,
+                    const gchar           *uri,
+                    const gchar           *key,
+                    GstTagList            *tag_list,
+                    const gchar           *tag)
 {
 	gboolean ret;
 	gdouble          n;
@@ -257,11 +252,11 @@ add_double_gst_tag (TrackerSparqlBuilder        *metadata,
 }
 
 static void
-add_fraction_gst_tag (TrackerSparqlBuilder         *metadata,
-                      const gchar       *uri,
-                      const gchar       *key,
-                      GstTagList        *tag_list,
-                      const gchar       *tag)
+add_fraction_gst_tag (TrackerSparqlBuilder  *metadata,
+                      const gchar           *uri,
+                      const gchar           *key,
+                      GstTagList            *tag_list,
+                      const gchar           *tag)
 {
 	gboolean ret;
 	GValue   n = {0,};
@@ -282,10 +277,10 @@ add_fraction_gst_tag (TrackerSparqlBuilder         *metadata,
 
 static void
 add_y_date_gst_tag (TrackerSparqlBuilder  *metadata,
-                    const gchar *uri,
-                    const gchar *key,
-                    GstTagList  *tag_list,
-                    const gchar *tag)
+                    const gchar           *uri,
+                    const gchar           *key,
+                    GstTagList            *tag_list,
+                    const gchar           *tag)
 {
 	GDate    *date;
 	gboolean  ret;
@@ -332,7 +327,7 @@ add_time_gst_tag (TrackerSparqlBuilder   *metadata,
 	if (ret) {
 		gint64 duration;
 
-		duration = llroundl ((long double) n / (long double) GST_SECOND);
+		duration = (n + (GST_SECOND / 2)) / GST_SECOND;
 
 		tracker_sparql_builder_predicate (metadata, key);
 		tracker_sparql_builder_object_int64 (metadata, duration);
@@ -427,9 +422,9 @@ get_embedded_album_art(MetadataExtractor *extractor)
 }
 
 static void
-extract_stream_metadata_tagreadbin (MetadataExtractor *extractor,
-                                    const gchar       *uri,
-                                    TrackerSparqlBuilder         *metadata)
+extract_stream_metadata_tagreadbin (MetadataExtractor     *extractor,
+                                    const gchar           *uri,
+                                    TrackerSparqlBuilder  *metadata)
 {
 	if (extractor->mime != EXTRACT_MIME_IMAGE) {
 		add_int_gst_tag (metadata, uri, "nfo:channels", extractor->tagcache, GST_TAG_CHANNEL);
@@ -501,7 +496,7 @@ extract_metadata (MetadataExtractor      *extractor,
                   gchar                 **scount)
 {
 	const gchar *temp;
-	gchar *s, *make = NULL, *model = NULL, *manuf = NULL, *make_model = NULL;
+	gchar *s, *make = NULL, *model = NULL, *manuf = NULL;
 	gboolean ret;
 	gint count;
 	gboolean needs_audio = FALSE;
@@ -540,7 +535,7 @@ extract_metadata (MetadataExtractor      *extractor,
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ARTIST, &artist_local);
 
 			if (artist_local) {
-				artist_uri = tracker_uri_printf_escaped ("urn:artist:%s", artist_local);
+				artist_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", artist_local);
 
 				tracker_sparql_builder_insert_open (preupdate, NULL);
 				tracker_sparql_builder_subject_iri (preupdate, artist_uri);
@@ -554,7 +549,7 @@ extract_metadata (MetadataExtractor      *extractor,
 			temp = tracker_coalesce_strip (2, performer, artist_local);
 
 			if (temp) {
-				performer_uri = tracker_uri_printf_escaped ("urn:artist:%s", temp);
+				performer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", temp);
 
 				tracker_sparql_builder_insert_open (preupdate, NULL);
 				tracker_sparql_builder_subject_iri (preupdate, performer_uri);
@@ -574,7 +569,7 @@ extract_metadata (MetadataExtractor      *extractor,
 			gst_tag_list_get_string (extractor->tagcache, GST_TAG_COMPOSER, &s);
 
 			if (s) {
-				composer_uri = tracker_uri_printf_escaped ("urn:artist:%s", s);
+				composer_uri = tracker_sparql_escape_uri_printf ("urn:artist:%s", s);
 
 				tracker_sparql_builder_insert_open (preupdate, NULL);
 
@@ -594,7 +589,11 @@ extract_metadata (MetadataExtractor      *extractor,
 
 		/* Audio */
 		s = NULL;
-		gst_tag_list_get_string (extractor->tagcache, GST_TAG_ALBUM, &s);
+
+		if (extractor->mime == EXTRACT_MIME_AUDIO) {
+			gst_tag_list_get_string (extractor->tagcache, GST_TAG_ALBUM, &s);
+		}
+
 		if (s) {
 			gboolean has_it;
 			guint count;
@@ -602,7 +601,7 @@ extract_metadata (MetadataExtractor      *extractor,
 
 			needs_audio = TRUE;
 
-			album_uri = tracker_uri_printf_escaped ("urn:album:%s", s);
+			album_uri = tracker_sparql_escape_uri_printf ("urn:album:%s", s);
 
 			tracker_sparql_builder_insert_open (preupdate, NULL);
 
@@ -741,6 +740,7 @@ extract_metadata (MetadataExtractor      *extractor,
 			tracker_sparql_builder_object (metadata, "nmm:Video");
 		} else if (!needs_audio) {
 			tracker_sparql_builder_object (metadata, "nfo:Image");
+			tracker_sparql_builder_object (metadata, "nmm:Photo");
 		}
 
 		s = NULL;
@@ -762,26 +762,31 @@ extract_metadata (MetadataExtractor      *extractor,
 		gst_tag_list_get_string (extractor->tagcache, GST_TAG_DEVICE_MAKE, &make);
 		gst_tag_list_get_string (extractor->tagcache, GST_TAG_DEVICE_MANUFACTURER, &manuf);
 
-		if (make && model && manuf) {
-			make_model = tracker_merge_const (" ", 3, manuf, make, model);
-		} else if (make && model) {
-			make_model = tracker_merge_const (" ", 2, make, model);
-		} else if (make && manuf) {
-			make_model = tracker_merge_const (" ", 2, manuf, make);
-		} else if (model && manuf) {
-			make_model = tracker_merge_const (" ", 2, manuf, model);
-		} else if (model) {
-			make_model = g_strdup (model);
-		} else if (make) {
-			make_model = g_strdup (make);
-		} else if (manuf) {
-			make_model = g_strdup (manuf);
-		}
+		if (make || model || manuf) {
+			gchar *equip_uri;
 
-		if (make_model) {
-			tracker_sparql_builder_predicate (metadata, "nfo:device");
-			tracker_sparql_builder_object_unvalidated (metadata, make_model);
-			g_free (make_model);
+			equip_uri = tracker_sparql_escape_uri_printf ("urn:equipment:%s:%s:",
+			                                              manuf ? manuf : make ? make : "",
+			                                              model ? model : "");
+
+			tracker_sparql_builder_insert_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, equip_uri);
+			tracker_sparql_builder_predicate (preupdate, "a");
+			tracker_sparql_builder_object (preupdate, "nfo:Equipment");
+
+			if (make || manuf) {
+				tracker_sparql_builder_predicate (preupdate, "nfo:manufacturer");
+				tracker_sparql_builder_object_unvalidated (preupdate, manuf ? manuf : make);
+			}
+			if (model) {
+				tracker_sparql_builder_predicate (preupdate, "nfo:model");
+				tracker_sparql_builder_object_unvalidated (preupdate, model);
+			}
+			tracker_sparql_builder_insert_close (preupdate);
+
+			tracker_sparql_builder_predicate (metadata, "nfo:equipment");
+			tracker_sparql_builder_object_iri (metadata, equip_uri);
+			g_free (equip_uri);
 		}
 
 		g_free (make);
@@ -857,6 +862,7 @@ extract_metadata (MetadataExtractor      *extractor,
 			tracker_sparql_builder_object (metadata, "nmm:Video");
 		} else if (!needs_audio) {
 			tracker_sparql_builder_object (metadata, "nfo:Image");
+			tracker_sparql_builder_object (metadata, "nmm:Photo");
 		}
 	}
 
@@ -1000,9 +1006,9 @@ add_tags (GstTagList *new_tags, MetadataExtractor *extractor)
 
 static gboolean
 poll_for_ready (MetadataExtractor *extractor,
-                GstState state,
-                gboolean ready_with_state,
-                gboolean ready_with_eos)
+                GstState           state,
+                gboolean           ready_with_state,
+                gboolean           ready_with_eos)
 {
 	gint64              timeout   = 5 * GST_SECOND;
 	GstBus             *bus       = extractor->bus;
@@ -1098,7 +1104,8 @@ poll_for_ready (MetadataExtractor *extractor,
 }
 
 static GstElement *
-create_decodebin_pipeline (MetadataExtractor *extractor, const gchar *uri)
+create_decodebin_pipeline (MetadataExtractor *extractor,
+                           const gchar *uri)
 {
 	GstElement *pipeline = NULL;
 
@@ -1148,7 +1155,8 @@ create_decodebin_pipeline (MetadataExtractor *extractor, const gchar *uri)
 }
 
 static GstElement *
-create_tagreadbin_pipeline (MetadataExtractor *extractor, const gchar *uri)
+create_tagreadbin_pipeline (MetadataExtractor *extractor,
+                            const gchar *uri)
 {
 	GstElement *pipeline = NULL;
 
@@ -1276,7 +1284,7 @@ tracker_extract_gstreamer (const gchar *uri,
 static void
 extract_gstreamer_audio (const gchar          *uri,
                          TrackerSparqlBuilder *preupdate,
-			 TrackerSparqlBuilder *metadata)
+                         TrackerSparqlBuilder *metadata)
 {
 	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_AUDIO);
 }
@@ -1284,7 +1292,7 @@ extract_gstreamer_audio (const gchar          *uri,
 static void
 extract_gstreamer_video (const gchar          *uri,
                          TrackerSparqlBuilder *preupdate,
-			 TrackerSparqlBuilder *metadata)
+                         TrackerSparqlBuilder *metadata)
 {
 	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_VIDEO);
 }
@@ -1292,7 +1300,7 @@ extract_gstreamer_video (const gchar          *uri,
 static void
 extract_gstreamer_image (const gchar          *uri,
                          TrackerSparqlBuilder *preupdate,
-			 TrackerSparqlBuilder *metadata)
+                         TrackerSparqlBuilder *metadata)
 {
 	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_IMAGE);
 }
@@ -1300,7 +1308,7 @@ extract_gstreamer_image (const gchar          *uri,
 static void
 extract_gstreamer_guess (const gchar          *uri,
                         TrackerSparqlBuilder *preupdate,
-			TrackerSparqlBuilder *metadata)
+                         TrackerSparqlBuilder *metadata)
 {
 	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_GUESS);
 }
