@@ -165,6 +165,7 @@ typedef struct {
 	const gchar *uri;
 	GString *content;
 	gboolean title_already_set;
+	gboolean generator_already_set;
 } MsOfficeXMLParserInfo;
 
 typedef struct {
@@ -1900,42 +1901,11 @@ xml_text_handler_document_data (GMarkupParseContext  *context,
                                 GError              **error)
 {
 	MsOfficeXMLParserInfo *info = user_data;
-	static gboolean found = FALSE;
-	static gboolean added = FALSE;
 
 	switch (info->tag_type) {
 	case MS_OFFICE_XML_TAG_WORD_TEXT:
-		if (info->style_element_present) {
-			if (atoi (text) == 0) {
-				tracker_text_validate_utf8 (text, -1, &info->content, NULL);
-				g_string_append_c (info->content, ' ');
-			}
-		}
-
-		if (info->preserve_attribute_present) {
-			gchar *keywords = g_strdup (text);
-			if (found) {
-				tracker_text_validate_utf8 (text, -1, &info->content, NULL);
-				g_string_append_c (info->content, ' ');
-				found = FALSE;
-			} else {
-				gchar *lasts;
-				gchar *keyw;
-
-				for (keyw = strtok_r (keywords, ",; ", &lasts);
-				     keyw;
-				     keyw = strtok_r (NULL, ",; ", &lasts)) {
-					if ((g_ascii_strncasecmp (keyw, "Table", 6) == 0) ||
-					    (g_ascii_strncasecmp (keyw, "Figure", 6) == 0) ||
-					    (g_ascii_strncasecmp (keyw, "Section", 7) == 0) ||
-					    (g_ascii_strncasecmp (keyw, "Index", 5) == 0)) {
-						found = TRUE;
-					}
-				}
-			}
-
-			g_free (keywords);
-		}
+		tracker_text_validate_utf8 (text, -1, &info->content, NULL);
+		g_string_append_c (info->content, ' ');
 		break;
 
 	case MS_OFFICE_XML_TAG_SLIDE_TEXT:
@@ -1994,10 +1964,13 @@ xml_text_handler_document_data (GMarkupParseContext  *context,
 	}
 
 	case MS_OFFICE_XML_TAG_GENERATOR:
-		if (!added) {
+		if (info->generator_already_set) {
+			g_warning ("Avoiding additional generator (%s) in MsOffice XML document '%s'",
+			           text, info->uri);
+		} else {
+			info->generator_already_set = TRUE;
 			tracker_sparql_builder_predicate (info->metadata, "nie:generator");
 			tracker_sparql_builder_object_unvalidated (info->metadata, text);
-			added = TRUE;
 		}
 		break;
 
